From: Ronnie Sahlberg <sahlb...@google.com>

Create a version of verify_refname_available that backends can use,
and add it to refs/refs-internal.h.

Signed-off-by: Ronnie Sahlberg <sahlb...@google.com>
Signed-off-by: David Turner <dtur...@twopensource.com>
Signed-off-by: Junio C Hamano <gits...@pobox.com>
Signed-off-by: Michael Haggerty <mhag...@alum.mit.edu>
---
 refs/files-backend.c | 65 ++++++++++++++++++++++++----------------------------
 refs/refs-internal.h | 20 ++++++++++++++++
 2 files changed, 50 insertions(+), 35 deletions(-)

diff --git a/refs/refs-internal.h b/refs/refs-internal.h
index 5f447d4..f6a79b9 100644
--- a/refs/refs-internal.h
+++ b/refs/refs-internal.h
@@ -93,6 +93,26 @@ enum peel_status {
  */
 enum peel_status peel_object(const unsigned char *name, unsigned char *sha1);
 
+/*
+ * Return 0 if a reference named refname could be created without
+ * conflicting with the name of an existing reference.
+ * Otherwise, return a negative value and write an explanation to err.
+ * If extras is non-NULL, it is a list of additional refnames with
+ * which refname is not allowed to conflict. If skip is non-NULL,
+ * ignore potential conflicts with refs in skip (e.g., because they
+ * are scheduled for deletion in the same operation). Behavior is
+ * undefined if the same name is listed in both extras and skip.
+ *
+ * Two reference names conflict if one of them exactly matches the
+ * leading components of the other; e.g., "foo/bar" conflicts with
+ * both "foo" and with "foo/bar/baz" but not with "foo/bar" or
+ * "foo/barbados".
+ *
+ * skip must be sorted.
+ */
+int verify_refname_available(const char *newname, struct string_list *extra,
+                            struct string_list *skip, struct strbuf *err);
+
 /**
  * Information needed for a single ref update. Set new_sha1 to the new
  * value or to null_sha1 to delete the ref. To check the old value
diff --git a/refs/files-backend.c b/refs/files-backend.c
index 20e2924..e0ae769 100644
--- a/refs/files-backend.c
+++ b/refs/files-backend.c
@@ -135,7 +135,7 @@ struct ref_dir {
  * presence of an empty subdirectory does not block the creation of a
  * similarly-named reference.  (The fact that reference names with the
  * same leading components can conflict *with each other* is a
  * separate issue that is regulated by verify_refname_available().)
  *
  * Please note that the name field contains the fully-qualified
  * reference (or subdirectory) name.  Space could be saved by only
@@ -720,25 +720,13 @@ static int nonmatching_ref_fn(struct ref_entry *entry, 
void *vdata)
 /*
  * Return 0 if a reference named refname could be created without
  * conflicting with the name of an existing reference in dir.
- * Otherwise, return a negative value and write an explanation to err.
- * If extras is non-NULL, it is a list of additional refnames with
- * which refname is not allowed to conflict. If skip is non-NULL,
- * ignore potential conflicts with refs in skip (e.g., because they
- * are scheduled for deletion in the same operation). Behavior is
- * undefined if the same name is listed in both extras and skip.
- *
- * Two reference names conflict if one of them exactly matches the
- * leading components of the other; e.g., "refs/foo/bar" conflicts
- * with both "refs/foo" and with "refs/foo/bar/baz" but not with
- * "refs/foo/bar" or "refs/foo/barbados".
- *
- * extras and skip must be sorted.
+ * See verify_refname_available for details.
  */
-static int verify_refname_available(const char *refname,
-                                   const struct string_list *extras,
-                                   const struct string_list *skip,
-                                   struct ref_dir *dir,
-                                   struct strbuf *err)
+static int verify_refname_available_dir(const char *refname,
+                                       const struct string_list *extras,
+                                       const struct string_list *skip,
+                                       struct ref_dir *dir,
+                                       struct strbuf *err)
 {
        const char *slash;
        int pos;
@@ -1942,9 +1930,12 @@ static struct ref_lock *lock_ref_sha1_basic(const char 
*refname,
                 */
                strbuf_git_path(&orig_ref_file, "%s", orig_refname);
                if (remove_empty_directories(&orig_ref_file)) {
+                       struct ref_dir *loose_refs;
+                       loose_refs = get_loose_refs(&ref_cache);
                        last_errno = errno;
-                       if (!verify_refname_available(orig_refname, extras, 
skip,
-                                                     
get_loose_refs(&ref_cache), err))
+                       if (!verify_refname_available_dir(orig_refname, extras,
+                                                         skip, loose_refs,
+                                                         err))
                                strbuf_addf(err, "there are still refs under 
'%s'",
                                            orig_refname);
                        goto error_return;
@@ -1957,8 +1948,9 @@ static struct ref_lock *lock_ref_sha1_basic(const char 
*refname,
        if (!refname) {
                last_errno = errno;
                if (last_errno != ENOTDIR ||
-                   !verify_refname_available(orig_refname, extras, skip,
-                                             get_loose_refs(&ref_cache), err))
+                   !verify_refname_available_dir(orig_refname, extras, skip,
+                                                 get_loose_refs(&ref_cache),
+                                                 err))
                        strbuf_addf(err, "unable to resolve reference %s: %s",
                                    orig_refname, strerror(last_errno));
 
@@ -1971,8 +1963,8 @@ static struct ref_lock *lock_ref_sha1_basic(const char 
*refname,
         * our refname.
         */
        if (is_null_oid(&lock->old_oid) &&
-           verify_refname_available(refname, extras, skip,
-                                    get_packed_refs(&ref_cache), err)) {
+           verify_refname_available_dir(refname, extras, skip,
+                                        get_packed_refs(&ref_cache), err)) {
                last_errno = ENOTDIR;
                goto error_return;
        }
@@ -2476,10 +2468,7 @@ static int rename_ref_available(const char *oldname, 
const char *newname)
        int ret;
 
        string_list_insert(&skip, oldname);
-       ret = !verify_refname_available(newname, NULL, &skip,
-                                       get_packed_refs(&ref_cache), &err)
-               && !verify_refname_available(newname, NULL, &skip,
-                                            get_loose_refs(&ref_cache), &err);
+       ret = !verify_refname_available(newname, NULL, &skip, &err);
        if (!ret)
                error("%s", err.buf);
 
@@ -2612,6 +2601,17 @@ static int commit_ref(struct ref_lock *lock)
        return 0;
 }
 
+int verify_refname_available(const char *newname, struct string_list *extra,
+                            struct string_list *skip, struct strbuf *err)
+{
+       struct ref_dir *packed_refs = get_packed_refs(&ref_cache);
+       struct ref_dir *loose_refs = get_loose_refs(&ref_cache);
+       return verify_refname_available_dir(newname, extra, skip,
+                                           packed_refs, err) ||
+               verify_refname_available_dir(newname, extra, skip,
+                                            loose_refs, err);
+}
+
 /*
  * Create a reflog for a ref.  If force_create = 0, the reflog will
  * only be created for certain refs (those for which
@@ -3327,8 +3327,6 @@ static int ref_present(const char *refname,
 int initial_ref_transaction_commit(struct ref_transaction *transaction,
                                   struct strbuf *err)
 {
-       struct ref_dir *loose_refs = get_loose_refs(&ref_cache);
-       struct ref_dir *packed_refs = get_packed_refs(&ref_cache);
        int ret = 0, i;
        int n = transaction->nr;
        struct ref_update **updates = transaction->updates;
@@ -3371,10 +3369,7 @@ int initial_ref_transaction_commit(struct 
ref_transaction *transaction,
                        die("BUG: initial ref transaction with old_sha1 set");
                if (verify_refname_available(update->refname,
                                             &affected_refnames, NULL,
-                                            loose_refs, err) ||
-                   verify_refname_available(update->refname,
-                                            &affected_refnames, NULL,
-                                            packed_refs, err)) {
+                                            err)) {
                        ret = TRANSACTION_NAME_CONFLICT;
                        goto cleanup;
                }
@@ -135,7 +135,7 @@ struct ref_dir {
  * similarly-named reference.  (The fact that reference names with the
  * same leading components can conflict *with each other* is a
- * separate issue that is regulated by verify_refname_available().)
+ * separate issue that is regulated by verify_refname_available_dir().)
  *
  * Please note that the name field contains the fully-qualified
-- 
2.6.2

--
To unsubscribe from this list: send the line "unsubscribe git" in
the body of a message to majord...@vger.kernel.org
More majordomo info at  http://vger.kernel.org/majordomo-info.html

Reply via email to